import { Maybe } from 'perhaps-ts';

/**
 * Represents a generic cache interface.
 * @template K - Type of the key.
 * @template V - Type of the value.
 */
declare interface Cache_2<K, V> {
    /**
     * Sets a value in the cache for the specified key.
     * @param {K} key - The key to set.
     * @param {V} value - The value to set.
     * @returns {this} The cache instance.
     */
    set(key: K, value: V): this;
    /**
     * Gets a value from the cache for the specified key.
     * @param {K} key - The key to get.
     * @returns {Maybe<V>} The value if found, otherwise Nothing.
     */
    get(key: K): Maybe<V>;
    /**
     * Checks if the cache has the specified key.
     * @param {K} key - The key to check.
     * @returns {boolean} True if the key exists, otherwise false.
     */
    has(key: K): boolean;
    /**
     * Deletes the value from the cache for the specified key.
     * @param {K} key - The key to delete.
     * @returns {boolean} True if the key was deleted, otherwise false.
     */
    delete(key: K): boolean;
    /**
     * Clears all values from the cache.
     */
    clear(): void;
    /**
     * Gets an iterable iterator of keys in the cache.
     * @returns {IterableIterator<K>} An iterator of keys.
     */
    keys(): IterableIterator<K>;
    /**
     * Gets an iterable iterator of values in the cache.
     * @returns {IterableIterator<V>} An iterator of values.
     */
    values(): IterableIterator<V>;
    /**
     * Gets an iterable iterator of entries (key-value pairs) in the cache.
     * @returns {IterableIterator<[K, V]>} An iterator of entries.
     */
    entries(): IterableIterator<[K, V]>;
    /**
     * Executes a callback function for each entry in the cache.
     * @param {(value: V, key: K) => void} callback - The callback function to execute.
     * @param {any} [thisArg] - Value to use as this when executing the callback.
     */
    forEach(callback: (value: V, key: K) => void, thisArg?: any): void;
    /**
     * Peeks at a value in the cache without updating its position.
     * @param {K} key - The key to peek at.
     * @returns {Maybe<V>} The value if found, otherwise Nothing.
     */
    peek(key: K): Maybe<V>;
    /**
     * Gets the number of entries in the cache.
     * @readonly
     * @type {number}
     */
    readonly size: number;
}
export { Cache_2 as Cache }

/**
 * Options for creating a cache.
 * @typedef {Object} CacheOptions@typedef {Object} CacheOptions
 * @property {number} maxSize - The maximum size of the cache.
 * @property {number} [maxAge] - The maximum age of a cache entry in milliseconds.
 * @property {(key: any, value: any) => void} [onEviction] - Callback function to execute when an entry is evicted.
 */
declare interface CacheOptions {
    maxSize: number;
    maxAge?: number;
    onEviction?: (key: any, value: any) => void;
}

/**
 * Represents a Least Frequently Used (LFU) cache.
 * @template K - Type of the key.
 * @template V - Type of the value.
 * @implements {Cache<K, V>}
 */
export declare class LFUCache<K, V> implements Cache_2<K, V> {
    private capacity;
    private cache;
    private freqMap;
    private minFreq;
    size: number;
    private maxAge;
    private onEviction?;
    /**
     * Creates an instance of LFUCache.
     * @param {CacheOptions} options - The options for the cache.
     */
    constructor(options: CacheOptions);
    /**
     * Updates the frequency of a node.
     * @private
     * @param {LFUNode<K, V>} node - The node to update.
     */
    private updateFrequency;
    /**
     * Sets a value in the cache for the specified key.
     * @param {K} key - The key to set.
     * @param {V} value - The value to set.
     * @param {{ maxAge?: number }} [options={}] - Options for the entry.
     * @returns {this} The cache instance.
     */
    set(key: K, value: V, { maxAge }?: {
        maxAge?: number;
    }): this;
    /**
     * Evicts the least frequently used node from the cache.
     * @private
     */
    private evict;
    /**
     * Deletes a node from the cache if it has expired.
     * @private
     * @param {K} key - The key to check.
     * @param {LFUNode<K, V>} item - The node to check.
     * @returns {boolean} True if the node was deleted, otherwise false.
     */
    private deleteIfExpired;
    /**
     * Gets a node from the cache if it has not expired, otherwise deletes it.
     * @private
     * @param {K} key - The key to get.
     * @param {LFUNode<K, V>} item - The node to get.
     * @returns {Maybe<V>} The value if found, otherwise Nothing.
     */
    private getOrDeleteIfExpired;
    /**
     * Peeks at the value in the cache for the specified key without updating its frequency.
     * @param {K} key - The key to peek.
     * @returns {Maybe<V>} The value if found, otherwise Nothing.
     */
    peek(key: K): Maybe<V>;
    /**
     * Gets the value in the cache for the specified key.
     * @param {K} key - The key to get.
     * @returns {Maybe<V>} The value if found, otherwise Nothing.
     */
    get(key: K): Maybe<V>;
    /**
     * Checks if the cache has the specified key.
     * @param {K} key - The key to check.
     * @returns {boolean} True if the key exists, otherwise false.
     */
    has(key: K): boolean;
    /**
     * Deletes the value from the cache for the specified key.
     * @param {K} key - The key to delete.
     * @returns {boolean} True if the key was deleted, otherwise false.
     */
    delete(key: K): boolean;
    /**
     * Clears all values from the cache.
     */
    clear(): void;
    /**
     * Gets an iterable iterator of entries in the cache in ascending order.
     * @private
     * @returns {IterableIterator<[K, LFUNode<K, V>]>} An iterator of entries.
     */
    private entriesAscendingInternal;
    /**
     * Emits eviction events for a set of cache entries.
     * @private
     * @param {Map<K, LFUNode<K, V>>} cache - The cache entries to evict.
     */
    private emitEvictions;
    /**
     * Resizes the cache to the new size.
     * @param {number} newSize - The new size of the cache.
     * @throws {TypeError} If the new size is not a positive number.
     */
    resize(newSize: number): void;
    /**
     * Gets an iterable iterator of keys in the cache.
     * @returns {IterableIterator<K>} An iterator of keys.
     */
    keys(): IterableIterator<K>;
    /**
     * Gets an iterable iterator of values in the cache.
     * @returns {IterableIterator<V>} An iterator of values.
     */
    values(): IterableIterator<V>;
    /**
     * Gets an iterable iterator of entries (key-value pairs) in the cache.
     * @returns {IterableIterator<[K, V]>} An iterator of entries.
     */
    [Symbol.iterator](): IterableIterator<[K, V]>;
    /**
     * Gets an iterable iterator of entries in the cache in ascending order.
     * @returns {IterableIterator<[K, V]>} An iterator of entries.
     */
    entriesAscending(): IterableIterator<[K, V]>;
    /**
     * Gets the maximum size of the cache.
     * @readonly
     * @type {number}
     */
    get maxSizeValue(): number;
    /**
     * Gets an iterable iterator of entries in the cache.
     * @returns {IterableIterator<[K, V]>} An iterator of entries.
     */
    entries(): IterableIterator<[K, V]>;
    /**
     * Executes a callback function for each entry in the cache.
     * @param {(value: V, key: K, map: Cache<K, V>) => void} callback - The callback function to execute.
     * @param {any} [thisArg=this] - Value to use as this when executing the callback.
     */
    forEach(callback: (value: V, key: K, map: Cache_2<K, V>) => void, thisArg?: this): void;
    /**
     * Gets the string tag representation of the cache.
     * @readonly
     * @type {string}
     */
    get [Symbol.toStringTag](): string;
}

/**
 * Represents a Least Recently Used (LRU) cache.
 * @template K - Type of the key.
 * @template V - Type of the value.
 * @implements {Cache<K, V>}
 */
export declare class LRUCache<K, V> implements Cache_2<K, V> {
    private capacity;
    private cache;
    private head;
    private tail;
    size: number;
    private maxAge;
    private onEviction?;
    /**
     * Creates an instance of LRUCache.
     * @param {CacheOptions} options - The options for the cache.
     */
    constructor(options: CacheOptions);
    /**
     * Moves a node to the head of the LRU list.
     * @private
     * @param {LRUNode<K, V>} node - The node to move.
     */
    private moveToHead;
    /**
     * Sets a value in the cache for the specified key.
     * @param {K} key - The key to set.
     * @param {V} value - The value to set.
     * @param {{ maxAge?: number }} [options={}] - Options for the entry.
     * @returns {this} The cache instance.
     */
    set(key: K, value: V, { maxAge }?: {
        maxAge?: number;
    }): this;
    /**
     * Deletes a value from the cache if it is expired.
     * @private
     * @param {K} key - The key to check.
     * @param {LRUNode<K, V>} item - The cache node to check.
     * @returns {boolean} True if the item was deleted, otherwise false.
     */
    private deleteIfExpired;
    /**
     * Gets a value from the cache if it is not expired, otherwise deletes it.
     * @private
     * @param {K} key - The key to get.
     * @param {LRUNode<K, V>} item - The cache node to get.
     * @returns {Maybe<V>} The value if found and not expired, otherwise Nothing.
     */
    private getOrDeleteIfExpired;
    /**
     * Peeks at a value in the cache without updating its position.
     * @param {K} key - The key to peek at.
     * @returns {Maybe<V>} The value if found, otherwise Nothing.
     */
    peek(key: K): Maybe<V>;
    /**
     * Gets a value from the cache for the specified key.
     * @param {K} key - The key to get.
     * @returns {Maybe<V>} The value if found, otherwise Nothing.
     */
    get(key: K): Maybe<V>;
    /**
     * Checks if the cache has the specified key.
     * @param {K} key - The key to check.
     * @returns {boolean} True if the key exists, otherwise false.
     */
    has(key: K): boolean;
    /**
     * Deletes the value from the cache for the specified key.
     * @param {K} key - The key to delete.
     * @returns {boolean} True if the key was deleted, otherwise false.
     */
    delete(key: K): boolean;
    /**
     * Clears all values from the cache.
     */
    clear(): void;
    /**
     * Gets an iterable iterator of entries in the cache in ascending order.
     * @private
     * @returns {IterableIterator<[K, LRUNode<K, V>]>} An iterator of entries.
     */
    private entriesAscendingInternal;
    /**
     * Emits eviction events for a set of cache entries.
     * @private
     * @param {Map<K, LRUNode<K, V>>} cache - The cache entries to evict.
     */
    private emitEvictions;
    /**
     * Resizes the cache to the new size.
     * @param {number} newSize - The new size of the cache.
     * @throws {TypeError} If the new size is not a positive number.
     */
    resize(newSize: number): void;
    /**
     * Gets an iterable iterator of keys in the cache.
     * @returns {IterableIterator<K>} An iterator of keys.
     */
    keys(): IterableIterator<K>;
    /**
     * Gets an iterable iterator of values in the cache.
     * @returns {IterableIterator<V>} An iterator of values.
     */
    values(): IterableIterator<V>;
    /**
     * Gets an iterable iterator of entries (key-value pairs) in the cache.
     * @returns {IterableIterator<[K, V]>} An iterator of entries.
     */
    [Symbol.iterator](): IterableIterator<[K, V]>;
    /**
     * Gets an iterable iterator of entries in the cache in descending order.
     * @returns {IterableIterator<[K, V]>} An iterator of entries.
     */
    entriesDescending(): IterableIterator<[K, V]>;
    /**
     * Gets an iterable iterator of entries in the cache in ascending order.
     * @returns {IterableIterator<[K, V]>} An iterator of entries.
     */
    entriesAscending(): IterableIterator<[K, V]>;
    /**
     * Gets the maximum size of the cache.
     * @readonly
     * @type {number}
     */
    get maxSizeValue(): number;
    /**
     * Gets an iterable iterator of entries in the cache.
     * @returns {IterableIterator<[K, V]>} An iterator of entries.
     */
    entries(): IterableIterator<[K, V]>;
    /**
     * Executes a callback function for each entry in the cache.
     * @param {(value: V, key: K, map: Cache<K, V>) => void} callback - The callback function to execute.
     * @param {any} [thisArg=this] - Value to use as this when executing the callback.
     */
    forEach(callback: (value: V, key: K, map: Cache_2<K, V>) => void, thisArg?: this): void;
    /**
     * Gets the string tag representation of the cache.
     * @readonly
     * @type {string}
     */
    get [Symbol.toStringTag](): string;
}

export { }
